"""
General utilities for the MPESA functions
"""

from __future__ import print_function
from .exceptions import *
from django_daraja.models import AccessToken
import requests
import json
from django.utils import timezone
from decouple import config, UndefinedValueError
import os
from requests import Response
import time
from django.conf import settings


class MpesaResponse(Response):
	response_description = ""
	error_code = None
	error_message = ''


def mpesa_response(r):
	"""
	Create MpesaResponse object from requests.Response object
	
	Arguments:
		r (requests.Response) -- The response to convert
	"""

	r.__class__ = MpesaResponse
	json_response = r.json()
	r.response_description = json_response.get('ResponseDescription', '')
	r.error_code = json_response.get('errorCode')
	r.error_message = json_response.get('errorMessage', '')
	return r


def mpesa_config(key):
	"""
	Get Mpesa configuration variable with the matching key
	
	Arguments:
		key (str) -- The configuration key

	Returns:
		str: Mpesa configuration variable with the matching key

	Raises:
		MpesaConfigurationException: Key not found
	"""

	value = getattr(settings, key, None)
	if value is None:
		try:
			value = config(key)
		except UndefinedValueError:
			# Check key in settings file
			raise MpesaConfigurationException('Mpesa environment not configured properly - ' + key + ' not found')

	return value


def api_base_url():
	"""
	Gets the base URL for making API calls

	Returns:
		The base URL depending on development environment (sandbox or production)

	Raises:
		MpesaConfigurationException: Environment not sandbox or production
	"""

	mpesa_environment = mpesa_config('MPESA_ENVIRONMENT')

	if mpesa_environment == 'sandbox':
		return 'https://sandbox.safaricom.co.ke/'
	elif mpesa_environment == 'production':
		return 'https://api.safaricom.co.ke/'
	else:
		raise MpesaConfigurationException('Mpesa environment not configured properly - MPESA_ENVIRONMENT should be sandbox or production')

def generate_access_token_request(consumer_key = None, consumer_secret = None):
	"""
	Make a call to OAuth API to generate access token
	
	Arguments:
		consumer_key (str) -- (Optional) The Consumer Key to use
		consumer_secret (str) -- (Optional) The Consumer Secret to use

	Returns:
		requests.Response: Response object with the response details

	Raises:
		MpesaConnectionError: Connection error
	"""

	url = api_base_url() + 'oauth/v1/generate?grant_type=client_credentials'
	consumer_key = consumer_key if consumer_key is not None else mpesa_config('MPESA_CONSUMER_KEY') 
	consumer_secret = consumer_secret if consumer_secret is not None else mpesa_config('MPESA_CONSUMER_SECRET')

	try:
		r = requests.get(url, auth=(consumer_key, consumer_secret))
	except requests.exceptions.ConnectionError:
		raise MpesaConnectionError('Connection failed')
	except Exception as ex:
		return ex.message
	
	return r

def generate_access_token():
	"""
	Parse generated OAuth access token, then updates database access token value

	Returns:
		AccessToken: The AccessToken object from the database

	Raises:
		MpesaError: Error generating access token
	"""

	r = generate_access_token_request()
	if r.status_code != 200:
		# Retry to generate access token
		r = generate_access_token_request()
		if r.status_code != 200:
			raise MpesaError('Unable to generate access token')

	token = r.json()['access_token']

	AccessToken.objects.all().delete()
	access_token = AccessToken.objects.create(token=token)

	return access_token

def mpesa_access_token():
	"""
	Generate access token if the current one has expired or if token is non-existent
	Otherwise return existing access token

	Returns:
		str: A valid access token
	"""

	# access_token = generate_access_token()

	access_token = AccessToken.objects.first()
	if access_token == None:
		# No access token found
		access_token = generate_access_token()
	else:
		delta = timezone.now() - access_token.created_at
		minutes = (delta.total_seconds()//60)%60
		print('minutes: ', minutes)
		if minutes > 50:
			# Access token expired
			access_token = generate_access_token()	
	
	return access_token.token

def format_phone_number(phone_number):
	"""
	Format phone number into the format 2547XXXXXXXX
	
	Arguments:
		phone_number (str) -- The phone number to format
	"""

	if len(phone_number) < 9:
		raise IllegalPhoneNumberException('Phone number too short')
	else:
		return '254' + phone_number[-9:]

def sleep(seconds, message=''):
	"""
	Sleep for the specified number of seconds
	
	Arguments:
		seconds (float) -- Number of seconds to sleep, can be a float
		message (str) -- (Optional) message to display
	"""

	print()
	print('===')
	print(message, end="")
	for i in range(seconds * 2):
		time.sleep(0.5)
		print('.', end="")
	print()
	print('===')
	print()


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models


class AccessToken(models.Model):
	token = models.CharField(max_length=30)
	created_at = models.DateTimeField(auto_now_add=True)

	class Meta:
		get_latest_by = 'created_at'

	def __str__(self):
		return self.token


#!/usr/bin/env python

from __future__ import absolute_import, print_function, unicode_literals

import os
import sys
import nose

def start(argv=None):
    sys.exitfunc = lambda: sys.stderr.write("Shutting down...\n")

    if argv is None:
        argv = [
            "nosetests", "--cover-branches", "--with-coverage",
            "--cover-erase", "--verbose",
            "--cover-package=django_daraja",
        ]

    nose.run_exit(argv=argv, defaultTest=os.path.abspath(os.path.dirname(__file__)))

if __name__ == "__main__":
    start(sys.argv)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.shortcuts import render
from django.http import HttpResponse, JsonResponse
from django.views.generic import View
from django_daraja.mpesa.core import *
from decouple import config
from datetime import datetime
cl = MpesaClient()

def index(request):

	return HttpResponse('Welcome to the home of daraja APIs')

def oauth_success(request):
	r = cl.access_token()
	return JsonResponse(r, safe=False)

def stk_push_success(request):
	phone_number = config('LNM_PHONE_NUMBER')
	amount = '1'
	account_reference = 'ABC001'
	transaction_desc = 'Description'
	callback_url = 'https://darajambili.herokuapp.com/express-payment'
	r = cl.stk_push(phone_number, amount, account_reference, transaction_desc, callback_url)
	return JsonResponse(r.response_description, safe=False)



# -*- coding: utf-8 -*-
"""
Test the Oauth API
"""

from __future__ import unicode_literals
from django.test import TestCase
from django_daraja.mpesa.core import MpesaClient
from django_daraja.mpesa.utils import *

class MpesaOauthTestCase(TestCase):

	def test_oauth_correct_credentials(self):
		'''
		Test correct credentials sent to oauth endpoint
		'''

		r = generate_access_token_request()
		self.assertEqual(r.status_code, 200)

	def test_oauth_wrong_credentials(self):
		'''
		Test wrong credentials sent to OAuth endpoint
		'''

		consumer_key = 'wrong_consumer_key'
		consumer_secret = 'wrong_consumer_secret'
		r = generate_access_token_request(consumer_key, consumer_secret)
		self.assertEqual(r.status_code, 400) # Unauthorized

	def test_access_token_valid(self):
		'''
		Test that access token is never older than 50 minutes	
		'''

		token = generate_access_token()
		delta = timezone.now() - token.created_at
		minutes = (delta.total_seconds()//60)%60
		self.assertLessEqual(minutes, 30)



# -*- coding: utf-8 -*-
'''
Test STK Push
'''

from django.test import TestCase
from django_daraja.mpesa.core import MpesaClient
from decouple import config
from django_daraja.mpesa.utils import *
from django_daraja.mpesa.exceptions import *

class MpesaStkPushTestCase(TestCase):

	cl = MpesaClient()

	def test_parse_stk_push_result_success(self):
		'''
		Test parse successful STK push result
		'''
		result = '{"Body":{"stkCallback":{"MerchantRequestID":"19918-3028728-2","CheckoutRequestID":"ws_CO_DMZ_264695293_11032019122104433","ResultCode":0,"ResultDesc":"The service request is processed successfully.","CallbackMetadata":{"Item":[{"Name":"Amount","Value":1.00},{"Name":"MpesaReceiptNumber","Value":"NCB9FJNAIT"},{"Name":"Balance"},{"Name":"TransactionDate","Value":20190311122121},{"Name":"PhoneNumber","Value":254719748260}]}}}}'
		data = self.cl.parse_stk_result(result)
		self.assertEqual(data.get('ResultDesc'), 'The service request is processed successfully.')


	def test_parse_stk_push_result_wrong_pin(self):
		'''
		Test parse STK push result with wrong user PIN
		'''
		result = '{"Body":{"stkCallback":{"MerchantRequestID":"14683-2472331-1","CheckoutRequestID":"ws_CO_DMZ_264800546_11032019141837240","ResultCode":2001,"ResultDesc":"[MpesaCB - ]The initiator information is invalid."}}}'
		data = self.cl.parse_stk_result(result)
		self.assertEqual(data.get('ResultDesc'), '[MpesaCB - ]The initiator information is invalid.')


	def test_parse_stk_push_result_cancelled(self):
		'''
		Test parse STK push result with request cancelled by user
		'''
		result = '{"Body":{"stkCallback":{"MerchantRequestID":"19919-3025233-1","CheckoutRequestID":"ws_CO_DMZ_401294732_11032019121308821","ResultCode":1032,"ResultDesc":"[STK_CB - ]Request cancelled by user"}}}'
		data = self.cl.parse_stk_result(result)
		self.assertEqual(data.get('ResultDesc'), '[STK_CB - ]Request cancelled by user')

	def test_parse_stk_push_result_timeout(self):
		'''
		Test parse STK push result after timeout
		'''
		result = '{"Body":{"stkCallback":{"MerchantRequestID":"19924-2088990-1","CheckoutRequestID":"ws_CO_DMZ_262987371_09032019203456095","ResultCode":1036,"ResultDesc":"[STK_CB - ]SMSC ACK timeout."}}}'
		data = self.cl.parse_stk_result(result)
		self.assertEqual(data.get('ResultDesc'), '[STK_CB - ]SMSC ACK timeout.')


# -*- coding: utf-8 -*-
'''
Test STK Push
'''

from django.test import TestCase
from django_daraja.mpesa.core import MpesaClient
from decouple import config
from django_daraja.mpesa.utils import *
from django_daraja.mpesa.exceptions import *

class MpesaStkPushTestCase(TestCase):

	cl = MpesaClient()

	def test_stk_push_success(self):
		'''
		Test successful STK push
		'''
		
		# Wait for a short while (to avoid SpikeArrest)
		sleep(5, 'STK push success')

		phone_number = config('LNM_PHONE_NUMBER')
		amount = 1
		account_reference = 'reference'
		transaction_desc = 'Description'
		callback_url = 'https://darajambili.herokuapp.com/express-payment'
		response = self.cl.stk_push(phone_number, amount, account_reference, transaction_desc, callback_url)
		self.assertEqual(response.response_description, 'Success. Request accepted for processing')

	def test_stk_push_empty_reference(self):
		'''
		Test that STK push with empty account reference raises MpesaInvalidParameterException
		'''

		with self.assertRaises(MpesaInvalidParameterException):
			phone_number = config('LNM_PHONE_NUMBER')
			amount = 1
			account_reference = ''
			transaction_desc = 'Description'
			callback_url = 'https://darajambili.herokuapp.com/express-payment'
			response = self.cl.stk_push(phone_number, amount, account_reference, transaction_desc, callback_url)

		
	def test_stk_push_empty_description(self):
		'''
		Test that STK push with empty description raises MpesaInvalidParameterException
		'''

		with self.assertRaises(MpesaInvalidParameterException):
			phone_number = config('LNM_PHONE_NUMBER')
			amount = 1000000
			account_reference = 'reference'
			transaction_desc = ''
			callback_url = 'https://darajambili.herokuapp.com/express-payment'
			response = self.cl.stk_push(phone_number, amount, account_reference, transaction_desc, callback_url)

	def test_stk_push_invalid_amount(self):
		'''
		Test that STK push with empty description raises MpesaInvalidParameterException
		'''

		with self.assertRaises(MpesaInvalidParameterException):
			phone_number = config('LNM_PHONE_NUMBER')
			amount = 1.5
			account_reference = 'reference'
			transaction_desc = 'Description'
			callback_url = 'https://darajambili.herokuapp.com/express-payment'
			response = self.cl.stk_push(phone_number, amount, account_reference, transaction_desc, callback_url)



import re
import os

try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='django-daraja',
    version='1.1.2',
    description='A python django library for interacting with the Safaricom MPESA Daraja API.',
    long_description=
'''
This is a django library based on the Safaricom MPESA daraja API. Use it for a simplified experience, spend less time setting up...

Read the full documentation at https://django-daraja.readthedocs.io

MPESA Daraja API documentation can be found at https://developer.safaricom.co.ke''',
    author='Martin Mogusu',
    author_email='martinmogusu@gmail.com',
    url='https://github.com/martinmogusu/django-daraja',
    download_url='https://github.com/martinmogusu/django-daraja.git',
    license='MIT License',
    packages=find_packages(exclude=['*.tests', '*.tests.*', 'tests.*', 'tests']),
    include_package_data=True,
    install_requires=[
        'Django>=1.11',
        'python-decouple',
        'requests'
    ],
    tests_require=[
        'nose',
        'coverage',
    ],
    zip_safe=False,
    test_suite='tests.runtests.start',
    classifiers=[
        'Operating System :: OS Independent',
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    keywords='mpesa django daraja finance mobile-money safaricom api',
    project_urls={
    'Documentation': 'https://django-daraja.readthedocs.io/',
    'Source': 'https://github.com/martinmogusu/django-daraja',
},
)


from django.conf.urls import url, include
from . import views

test_patterns = [
	url(r'^$', views.index, name='django_daraja_index'),
	url(r'^oauth/success', views.oauth_success, name='test_oauth_success'),
	url(r'^stk-push/success', views.stk_push_success, name='test_oauth_success'),
]

urlpatterns = [
	url(r'^$', views.index, name='index'),
	url(r'^tests/', include(test_patterns)),
]

